@@ -14,7 +14,7 @@ require 'amstd/rbparams'
 
 module Getdep
 
-  DL_EXT_EXP = /\.#{RubyParams::DL_EXT}\z/o
+  DL_EXT_EXP = /\.#{RubyParams::DLEXT}\z/o
 
   def getdep( fn, done = {} )
     unless DL_EXT_EXP === fn then   # don't grep binary file
