@@ -1,43 +1,48 @@
-  
+
   MAILP_DEBUG = false
 
   def initialize
-    @scanner = nil
     self.debug = MAILP_DEBUG
   end
 
-
   def debug=( flag )
     @yydebug = flag && DEBUG_PARSER
     @scanner_debug = flag
   end
 
-
   def debug
     @yydebug
   end
 
 
-  def Mailp.parse( header, str )
-    new.parse( header, str )
+  def Mailp.parse( str, obj, ident )
+    new.parse( str, obj, ident )
   end
 
 
-  def parse( header, str )
+  def parse( str, obj, ident )
     return if /\A\s*\z/ === str
 
-    @field = header
+    @field = obj
 
-    case header
-    when MsgidH then msgid_parse str
-    when RefH   then refs_parse str
+    case obj
+    when MsgidH
+      msgid_parse str
+    when RefH
+      refs_parse str
     else
-      @scanner = MailScanner.new( str, header )
-      @scanner.debug = @scanner_debug
+      unless ident then
+        ident = obj.type.name.split('::')[-1].to_s
+        cmt = []
+        obj.comments.replace cmt
+      else
+        cmt = nil
+      end
 
-      sim = header.type.to_s
-      sim = sim.split('::')[-1].to_s.intern
-      @first = [sim, nil]
+      @scanner = MailScanner.new( str, ident, cmt )
+      @scanner.debug = @scanner_debug
+      @first = [ ident.intern, ident ]
+      @pass_array = [nil, nil]
 
       do_parse
     end
@@ -48,17 +53,16 @@
 
 
   def next_token
-    if s = @first then
+    if @first then
+      ret = @first
       @first = nil
+      ret
     else
-      s = @scanner.scan
+      @scanner.scan @pass_array
     end
-
-    s
   end
-
   
-  def on_error( t, val, vstack )
+  def on_error( tok, val, vstack )
     raise ParseError,
       "\nparse error in '#{@field.name}' header, on token #{val.inspect}"
   end
@@ -67,9 +71,9 @@
   def refs_parse( str )
     arr = []
 
-    while MsgidH::MSGID === str do
-      p = $`
-      m = $&
+    while ::TMail::MSGID === str do
+      p   = $`
+      m   = $&
       str = $'
 
       p.strip!
@@ -82,13 +86,11 @@
     @field.refs.replace arr
   end
 
-
   def msgid_parse( str )
-    if MsgidH::MSGID === str then
+    if ::TMail::MSGID === str then
       @field.msgid = $&
     else
-      raise ParseError,
-        "wrong Message-ID format: #{str}"
+      raise ParseError, "wrong Message-ID format: #{str}"
     end
   end
 
